package at.ac.uibk.gitsearch.es.model;

import java.util.List;
import java.util.stream.Collectors;

import org.codeability.sharing.plugins.api.search.UserProvidedMetadataDTO.Person;

import com.fasterxml.jackson.annotation.JsonEnumDefaultValue;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonValue;

import at.ac.uibk.gitsearch.service.dto.MetadataUserDTO;

@JsonInclude(JsonInclude.Include.NON_NULL)
public class ExerciseInfo {

    private String metadataVersion;

    private String title;

    private String license;

    private String description;

    private List<String> keyword;

    private List<String> format;

    private String structure;

    private List<String> programmingLanguage;

    private List<String> language;

    private String learningResourceType;

    private String identifier;

    private ExerciseDifficulty difficulty;

    private List<MetadataUserDTO> creator;

    private List<MetadataUserDTO> contributor;

    private List<MetadataUserDTO> publisher;

    public ExerciseInfo() {
    }

    public String getMetadataVersion() {
        return metadataVersion;
    }

    public void setMetadataVersion(String metadataVersion) {
        this.metadataVersion = metadataVersion;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getLicense() {
        return license;
    }

    public void setLicense(String license) {
        this.license = license;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public List<String> getKeyword() {
        return keyword;
    }

    public void setKeyword(List<String> keyword) {
        this.keyword = keyword;
    }

    public List<String> getFormat() {
        return format;
    }

    public void setFormat(List<String> format) {
        this.format = format;
    }

    public String getStructure() {
        return structure;
    }

    public void setStructure(String structure) {
        this.structure = structure;
    }

    public List<String> getProgrammingLanguage() {
        return programmingLanguage;
    }

    public void setProgrammingLanguage(List<String> programmingLanguage) {
        this.programmingLanguage = programmingLanguage;
    }

    public List<String> getLanguage() {
        // re-map to metadata conform values
        if (language != null) {
            return language.stream().map(entry -> {
                switch (entry) {
                    case "english":
                        return "en";
                    case "german":
                        return "de";
                }
                return entry;
            }).collect(Collectors.toList());
        } else {
            return null;
        }
    }

    public void setLanguage(List<String> language) {
        this.language = language;
    }

    public ExerciseDifficulty getDifficulty() {
        return difficulty;
    }

    public void setDifficulty(ExerciseDifficulty difficulty) {
        this.difficulty = difficulty;
    }

    public String getLearningResourceType() {
        return learningResourceType;
    }

    public void setLearningResourceType(String learningResourceType) {
        this.learningResourceType = learningResourceType;
    }

    public String getIdentifier() {
        return identifier;
    }

    public void setIdentifier(String identifier) {
        this.identifier = identifier;
    }

    @JsonProperty("creator")
    public List<MetadataUserDTO> getCreator() {
        return creator;
    }

    public void setCreator(List<MetadataUserDTO> creator) {
        this.creator = creator;
    }

    @JsonProperty("publisher")
    public List<MetadataUserDTO> getPublisher() {
        return publisher;
    }

    public void setPublisher(List<MetadataUserDTO> publisher) {
        this.publisher = publisher;
    }

    @JsonProperty("contributor")
    public List<MetadataUserDTO> getContributor() {
        return contributor;
    }

    public void setContributor(List<MetadataUserDTO> contributor) {
        this.contributor = contributor;
    }

    @Override
    public String toString() {
        return "ExerciseInfo{" +
            "metadataVersion='" + metadataVersion + '\'' +
            ", title='" + title + '\'' +
            ", license='" + license + '\'' +
            ", keyword=" + keyword +
            ", format=" + format +
            ", structure='" + structure + '\'' +
            ", programmingLanguage=" + programmingLanguage +
            ", language=" + language +
            ", learningResourceType=" + learningResourceType +
            ", identifier=" + identifier +
            ", difficulty=" + difficulty +
            ", creator=" + creator +
            ", publisher=" + publisher +
            ", contributor=" + contributor +
            '}';
    }

    @JsonFormat(with = JsonFormat.Feature.ACCEPT_CASE_INSENSITIVE_PROPERTIES)
    public enum ExerciseDifficulty {
        SIMPLE("simple"),
        MEDIUM("medium"),
        ADVANCED("advanced"),
        @JsonEnumDefaultValue NONE("none");

        ExerciseDifficulty(String s) {}

        @JsonValue
        public String jacksonCustomMapping() {
            return name().toLowerCase();
        }

        @Override
        public String toString() {
            return name().toLowerCase();
        }
    }

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((contributor == null) ? 0 : contributor.hashCode());
		result = prime * result + ((publisher == null) ? 0 : publisher.hashCode());
		result = prime * result + ((creator == null) ? 0 : creator.hashCode());
		result = prime * result + ((description == null) ? 0 : description.hashCode());
		result = prime * result + ((difficulty == null) ? 0 : difficulty.hashCode());
		result = prime * result + ((format == null) ? 0 : format.hashCode());
		result = prime * result + ((keyword == null) ? 0 : keyword.hashCode());
		result = prime * result + ((language == null) ? 0 : language.hashCode());
		result = prime * result + ((license == null) ? 0 : license.hashCode());
		result = prime * result + ((metadataVersion == null) ? 0 : metadataVersion.hashCode());
		result = prime * result + ((programmingLanguage == null) ? 0 : programmingLanguage.hashCode());
		result = prime * result + ((structure == null) ? 0 : structure.hashCode());
		result = prime * result + ((title == null) ? 0 : title.hashCode());
        result = prime * result + ((learningResourceType == null) ? 0 : learningResourceType.hashCode());
        result = prime * result + ((identifier == null) ? 0 : identifier.hashCode());
        return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof ExerciseInfo))
			return false;
		ExerciseInfo other = (ExerciseInfo) obj;
		if (contributor == null) {
			if (other.contributor != null)
				return false;
		} else if (!contributor.equals(other.contributor))
			return false;
		if (creator == null) {
			if (other.creator != null)
				return false;
		} else if (!creator.equals(other.creator))
			return false;
        if (publisher == null) {
            if (other.publisher != null)
                return false;
        } else if (!publisher.equals(other.publisher))
            return false;
		if (description == null) {
			if (other.description != null)
				return false;
		} else if (!description.equals(other.description))
			return false;
		if (difficulty != other.difficulty)
			return false;
		if (format == null) {
			if (other.format != null)
				return false;
		} else if (!format.equals(other.format))
			return false;
		if (keyword == null) {
			if (other.keyword != null)
				return false;
		} else if (!keyword.equals(other.keyword))
			return false;
		if (language == null) {
			if (other.language != null)
				return false;
		} else if (!language.equals(other.language))
			return false;
		if (license == null) {
			if (other.license != null)
				return false;
		} else if (!license.equals(other.license))
			return false;
		if (metadataVersion == null) {
			if (other.metadataVersion != null)
				return false;
		} else if (!metadataVersion.equals(other.metadataVersion))
			return false;
		if (programmingLanguage == null) {
			if (other.programmingLanguage != null)
				return false;
		} else if (!programmingLanguage.equals(other.programmingLanguage))
			return false;
		if (structure == null) {
			if (other.structure != null)
				return false;
		} else if (!structure.equals(other.structure))
			return false;
		if (title == null) {
			if (other.title != null)
				return false;
		} else if (!title.equals(other.title))
			return false;
        if (learningResourceType == null) {
            if (other.learningResourceType != null)
                return false;
        } else if (!learningResourceType.equals(other.learningResourceType))
            return false;
        if (identifier == null) {
            if (other.identifier != null)
                return false;
        } else if (!identifier.equals(other.identifier))
            return false;
		return true;
	}




}
